# Διάφορα στυλ για αναφορές pdf

from reportlab.lib import colors
from reportlab.pdfgen import canvas
from reportlab.lib.units import cm, mm 
from reportlab.lib.pagesizes import A4
from reportlab.platypus import SimpleDocTemplate, Table, TableStyle, PageBreak, Paragraph, Spacer
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.pdfbase.pdfmetrics import registerFont, registerFontFamily

# add pagenumbers to a report lab generated pdf page - it presupposes that a variable named last_page is defined and equal with the total number of pages in the document
# called in doc.build( elements, onFirstPage=addPageNumber, onLaterPages=addPageNumber)
def setlast(last):
    global last_page
    last_page = last

def addPageNumber(canvas, doc):
    canvas.setFont('DejaVu', 8)
    page_num = canvas.getPageNumber()
    text = "%s / %s" % (page_num,last_page)
    canvas.drawRightString(285*mm, 200*mm, text)

# στυλ για τη μορφοποίηση πινάκων του reportlab
# TODO να γίνουν πιο περιγραφικά τα ονόματα
def style(id):
    styles = getSampleStyleSheet()

    pdfmetrics.registerFont(TTFont('DejaVu', 'DejaVuSerif.ttf'))
    pdfmetrics.registerFont(TTFont('DejaVu-Bold', 'DejaVuSerif-Bold.ttf'))
    pdfmetrics.registerFont(TTFont('VeraIt', 'DejaVuSerif-Italic.ttf'))
    pdfmetrics.registerFont(TTFont('VeraBI', 'DejaVuSerif-BoldItalic.ttf'))
    pdfmetrics.registerFont(TTFont('Vera', 'DejaVuSerif.ttf'))
    registerFontFamily('DejaVu',normal='DejaVu',bold='DejaVu-Bold',italic='VeraIt',boldItalic='VeraBI')

    match id: 
        case "taxi tmima header":
            return ParagraphStyle('normal style',
                fontName='DejaVu',
                spaceAfter=2,
                parent=styles['Normal'],
                )
        case "1":
            return TableStyle([('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),('BOX', (0,0), (-1,0), 1.25, colors.black),('BOX', (0,0), (-1,-1), 1.25, colors.black),('FONT', (0, 1), (-1, -1), 'DejaVu'), ('FONT', (0, 0), (-1, 0), 'DejaVu-Bold'), ('FONTSIZE', (0,1), (-1,-1), 10), ('ALIGN', (0, 0), (1, -1), 'CENTER') ])
        case "Align Justified":
            return ParagraphStyle('normal style',
                parent=styles['Normal'],
                fontName='DejaVu',
                alignment=1,
                )
        case "Align Left":
            return ParagraphStyle('normal style',
                parent=styles['Normal'],
                fontName='DejaVu',
                alignment=0,
                )
        case "Headings (1)":
            return ParagraphStyle('heading1 style',
                fontName="DejaVu-Bold",
                parent=styles['Heading1'],
                )
        case "Table-Year-Date":
            return TableStyle([('FONT', (0, 0), (-1, -1), 'DejaVu'),('FONTSIZE', (0,0), (-1,-1), 8), ('ALIGN',(0,0),(-1,-1),'RIGHT'),])
        case "Tbl4":
            return TableStyle([('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),('BOX', (0,0), (-1,0), 1.25, colors.black),('BOX', (0,0), (-1,-1), 1.25, colors.black),('FONT', (0, 1), (-1, -1), 'DejaVu'), ('FONT', (0, 0), (-1, 0), 'DejaVu-Bold'), ('FONTSIZE', (0,1), (-1,-1), 10), ('ALIGN', (0, 0), (0, -1), 'CENTER'), ('ALIGN', (1, 0), (2, -1), 'LEFT'), ('ALIGN', (3, 0), (5, 0), 'CENTER'), ('ALIGN', (3, 1), (5, -1), 'LEFT'), ('VALIGN', (0, 0), (-1, -1), 'MIDDLE') ])
        case "TblHeaders1":
            return TableStyle([('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),('BOX', (0,0), (-1,0), 1.25, colors.black),('BOX', (0,0), (-1,-1), 1.25, colors.black),('FONT', (0, 1), (-1, -1), 'DejaVu'), ('FONT', (0, 0), (-1, 0), 'DejaVu-Bold'), ('FONTSIZE', (0,1), (-1,-1), 10), ('ALIGN', (0, 0), (0, -1), 'CENTER'), ('ALIGN', (1, 0), (2, -1), 'LEFT'), ('ALIGN', (3, 0), (3, -1), 'CENTER'), ('VALIGN', (0, 0), (-1, -1), 'MIDDLE') ])



import re

APOF_ID = "cbpCriteria_chkIsForAdditionalSpeciality_S_D"
APOF_STATES = {
    "APOF":"dxWeb_edtCheckBoxChecked",
    "NO_APOF":"dxWeb_edtCheckBoxUnChecked",
    "BOTH":"dxWeb_edtCheckBoxGreyed",
    }
GRADES = ['Α','Β','Γ']


class ShortCode:
    report_base = "https://app.myschool.sch.gr/Report.bas.aspx?rpt="
    def __init__(self,url,basename):
        self.url = self.report_base + url
        self.basename = basename

# όνομα αναφοράς - αριθμός url - όνομα αρχείου (χωρίς κατάληξη)
report_shortcodes = {
    "Γενικά στοιχεία εργαζόμενων":ShortCode("25","rptList"),
    "Γενικά στοιχεία μαθητών":ShortCode("64","rptList"),
    "Κατάσταση Παιδαγωγικών μέτρων":ShortCode("8","rptList"),
    "Τμήματα μαθητή":ShortCode("178","rptList"),
    "Δελτίο κίνησης - 1":ShortCode("175","175"),
    "Αναλυτική κατάσταση απουσιών":ShortCode("2","rptList")
}

page_urls = {
    "Κατάλογος μαθητών":"https://app.myschool.sch.gr/Student.list.aspx?action=list&clear=1",
    "Απουσίες μαθητή":"https://app.myschool.sch.gr/Student.edit.leav.aspx",
}

absenceJustificationTypeId = {
    "0":0,            #<empty string> debugger eval code:6:13
    "άρθρο 21":1279,  # Περ. αρθ. 21 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024) debugger eval code:6:13
    "άρθρο 23":1328,  # Περ. αρθ. 23 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024) debugger eval code:6:13
    "άρθρο 25":1377,  # Περ. αρθ. 25 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024) debugger eval code:6:13
    "γιατρός":1426,   # Ιατρική Βεβαίωση {Αρθ. 26 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024)} debugger eval code:6:13
    "κηδεμόνας":1475, # Υπεύθυνη Δήλωση Κηδεμόνα {(Αρθ. 26 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024} debugger eval code:6:13
    "γρίπη":1531,     # Λόγω εποχικών ιογενών αναπνευστικών λοιμώξεων
}

absenceTypeId = {
    "0":0,         #  <empty string>
    "Ωριαία":630,  # Προφορική παρατήρηση με ωριαία απομάκρυνση
    "1 μέρα":698,  # Αποβολή από τα μαθήματα μιας (01) ημέρας
    "2 μέρες":747, # Αποβολή από τα μαθήματα δυο (02) ημερών
    "3 μέρες":796, # Αποβολή από τα μαθήματα τριών (03) ημερών
    "4 μέρες":845, # Αποβολή από τα μαθήματα τεσσάρων (04) ημερών
    "5 μέρες":894, # Αποβολή από τα μαθήματα πέντε (05) ημερών
}

class js_scripts:
    apousies_prep_script = re.sub(r"\s+"," ","""
    window.FIELDS = {
      DateFrom: "DateFrom",
      Qty: "Qty",
      JustifiedQty: "JustifiedQty",
      StudentAbsenceJustificationTypeId: "StudentAbsenceJustificationTypeId",
      StudentAbsenceTypeId: "StudentAbsenceTypeId",
    };

    window.autoEditQueue = {
      _queue: [],
      _busy: false,

      enqueue(rowIndex, values) {
        return new Promise(resolve => {
            this._queue.push({ rowIndex, values, resolve });
            this._process();
        });
      },

      async _process() {
        if (this._busy || this._queue.length === 0) return;
        this._busy = true;

        const { rowIndex, values, resolve } = this._queue.shift();
        await this._run(rowIndex, values);
        resolve(); 

        this._busy = false;
        this._process();
      },

      _run(rowIndex, values) {
        return new Promise(resolve => {
            function onUpdateFinished(s, e) {
                s.EndCallback.RemoveHandler(onUpdateFinished);
                resolve();
            }

            function onEditFormReady(s, e) {
                s.EndCallback.RemoveHandler(onEditFormReady);

                Object.values(FIELDS).forEach(field => {
                    if (values[field] != null) s.SetEditValue(field, values[field]);
                });

                s.EndCallback.AddHandler(onUpdateFinished);
                s.UpdateEdit();
            }

            gridStudentAbsenceSummary.EndCallback.AddHandler(onEditFormReady);

            if (rowIndex === -1) {
                gridStudentAbsenceSummary.AddNewRow();
            } else {
                gridStudentAbsenceSummary.StartEditRow(rowIndex);
            }
        });
    }
};

window.startAutoEdit = function(rowIndex, values) {
    return autoEditQueue.enqueue(rowIndex, values);
};
""")
    



# διάφορες μικρές βοηθητικές συναρτήσεις
import datetime, time, configparser
from pathlib import Path
import argparse
import logging
from babel.dates import format_datetime, format_date
from openpyxl.utils.cell import range_boundaries
from openpyxl import load_workbook
from pandas import read_excel
from io import BytesIO
from collections import Counter
from beaupy import confirm, prompt, select, select_multiple
from beaupy.spinners import Spinner, DOTS
from rich.console import Console
from importlib.metadata import version, PackageNotFoundError

try:
    version = version("gr_sch_helper")
except PackageNotFoundError:
    version = "unknown"

_config = configparser.ConfigParser()
console = Console()
logger = None
GRADES = ['Α','Β','Γ']

def load_config(path):
    global _config 
    _config.read(path, 'UTF-8')

def set_config(section, name, value):
    global _config
    if not _config.has_section(section):
        _config.add_section(section)
    _config.set(section, name, value)

def get_config(section, option, fallback=None):
    return _config.get(section, option, fallback=fallback)

def set_rconfig(name, value):
    set_config('runtime', name, value)
    
def get_rconfig(name, fallback=None):
    return get_config('runtime',name, fallback=fallback)

def test_fun():
    print(f"Howdy shortM!")

def standard_args(arglist=[]):
    parser = argparse.ArgumentParser(add_help=False, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-h', '--help', action="help", help="εμφανίζει το παρόν μήνυμα βοήθειας και σταματά")
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s - έκδοση:{version}', help="εμφανίζει την έκδοση του προγράμματος και σταματά")
    parser.add_argument('-b', '--batch', action='store_true', help="αυτόματη λειτουργία")
    parser.add_argument('-d', '--debug', action='store_true', help='εμφάνιση μυνημάτων αποσφαλμάτωσης και παράθυρου firefox')
    return parser   

def standard_logging(name, args):
    logger = logging.getLogger(name)
    logger.propagate = False  # avoid double logging if root logger exists

    # Create a single stream handler
    handler = logging.StreamHandler()
    formatter = logging.Formatter('%(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    if args.batch and not args.debug:
        logger.setLevel(logging.WARNING)
        handler.setLevel(logging.WARNING)
        set_rconfig("show_spinner", "False")
    else:
        logger.setLevel(logging.INFO)
        handler.setLevel(logging.INFO)
        set_rconfig("show_spinner", "True")
        
        # Σε λειτουργία διόρθωσης προβλημάτων εμφανίζουμε το παράθυρο του firefox                                                                   
    headless = False if args.debug else True
    global schlogger
    schlogger = logger
    return logger, headless

def get_school_year():
    month = datetime.datetime.now().month
    year = datetime.datetime.now().year
    if month < 9:
        return f"{year-1} - {year}"
    else:
        return f"{year} - {year+1}"

def get_date_long():
    date = datetime.datetime.now().date()
    return format_date(date, 'EEEE, d MMMM yyyy', locale='el_GR')

def get_date_filename():
    date = datetime.datetime.now()
    return format_datetime(date, 'yyyy.MM.dd-HH-mm', locale='el_GR')

class ShortCode:
    report_base = "https://app.myschool.sch.gr/Report.bas.aspx?rpt="
    def __init__(self,url,basename):
        self.url = self.report_base + url
        self.basename = basename

def get_folder_files(ext_list):
    file_list = []
    for workfilepath in Path.cwd().iterdir():
        workfile = workfilepath.name
        ext = workfilepath.suffix
        if ext in ext_list:
            file_list.append(workfile)
    file_list.sort(reverse=True)
    return file_list

def set_option_select(msg,option,choice_list):
    if type(choice_list[0]) == type("string"):
        descriptions = choice_list
        choices = choice_list
    elif type(choice_list[0]) == type(("tup","le")):
        descriptions = [x[0] for x in choice_list]
        choices =   [x[1] for x in choice_list]
    console.print(msg)
    ind = select (descriptions, cursor="->", cursor_style="cyan", return_index = True)
    return choices[ind]
    
def load_orologio(path):
    wb = load_workbook(path)
    ws = wb.active
    for cell_group in list(ws.merged_cells.ranges):
        min_col, min_row, max_col, max_row = range_boundaries(str(cell_group))
        top_left_cell_value = ws.cell(row=min_row, column=min_col).value
        ws.unmerge_cells(str(cell_group))
        for row in ws.iter_rows(min_col=min_col, min_row=min_row, max_col=max_col, max_row=max_row):
            for cell in row:
                cell.value = top_left_cell_value
    buffer = BytesIO()
    wb.save(buffer)
    buffer.seek(0)
    return read_excel(buffer,na_filter=False)

        
# όνομα αναφοράς - αριθμός url - όνομα αρχείου (χωρίς κατάληξη)
report_shortcodes = {
    "Γενικά στοιχεία εργαζόμενων":ShortCode("25","rptList"),
    "Γενικά στοιχεία μαθητών":ShortCode("64","rptList"),
    "Κατάσταση Παιδαγωγικών μέτρων":ShortCode("8","rptList"),
    "Τμήματα μαθητή":ShortCode("178","rptList"),
    "Δελτίο κίνησης - 1":ShortCode("175","175"),
    "Αναλυτική κατάσταση απουσιών":ShortCode("2","rptList")
}

def countme(list):
    return Counter(list)

def expand_lessons(lessons):
    expanded = []
    for lesson, n in zip(lessons[0::2], lessons[1::2]):
        expanded.extend([lesson] * int(n))
    return expanded

def sorted_list_diff(lista,listb):
    a_not_b = []
    b_not_a = []
    
    i = j = 0
    while i < len(lista) and j < len(listb):
        if lista[i] < listb[j]:
            a_not_b.append(lista[i])
            i += 1
        elif lista[i] > listb[j]:
            b_not_a.append(listb[j])
            j += 1
        else:
            i += 1
            j += 1
    a_not_b.extend(lista[i:])
    b_not_a.extend(listb[j:])
    return (a_not_b, b_not_a)

def empty_spinner():
    class Dummy:
        def start(self): pass
        def stop(self): pass
    return Dummy()

def spin(msg):
    show_spinner = get_rconfig("show_spinner") == "True"
    if show_spinner:
        spinner = Spinner(DOTS, f"{msg}")
        spinner.start()
    else:
        spinner = empty_spinner()
    return spinner
    
def stop(spinner, msg):
    spinner.stop()
    schlogger.info(f"✓ {msg}")

def assure_deletion(path, timeout=20, interval=0.5):
    path = Path(path)
    end = time.time() + timeout
    while time.time() < end:
        if not path.exists():
            return
        time.sleep(interval)
    raise TimeoutError(f"File still exists: {path}")




# library για σύνδεση και κατέβασμα αρχείων από το myschool
# στόχος είναι το κάθε script να μπορεί να κάνει πχ:
# driver = get_driver(headless=True)
# download_report(driver, "Γενική τμημάτων","με μητέρα","target_file.xls")

import pathlib, time, os, re
from contextlib import contextmanager

from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import (
    TimeoutException,
    NoSuchElementException,
    StaleElementReferenceException
)
from selenium.webdriver.firefox.service import Service as FirefoxService
from gr_sch_helper import sch, constants
from gr_sch_helper.sch import get_config

# Τα id του mySchool για την κάθε τάξη
grade_links = {
    "ΑΛ":"ctl00_ContentData_lstLevel_6043_D",
    "ΒΛ":"ctl00_ContentData_lstLevel_6044_D",
    "ΓΛ":"ctl00_ContentData_lstLevel_6045_D",
}

report_shortcodes = constants.report_shortcodes
page_urls = constants.page_urls

report_types = {
    "csv":"ReportToolbar1_Menu_ITCNT13_SaveFormat_DDD_L_LBI7T0",
}

absenseJustificationTypeId = {
    "0":0,            #<empty string> debugger eval code:6:13
    "άρθρο 21":1279,  # Περ. αρθ. 21 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024) debugger eval code:6:13
    "άρθρο 23":1328,  # Περ. αρθ. 23 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024) debugger eval code:6:13
    "άρθρο 25":1377,  # Περ. αρθ. 25 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024) debugger eval code:6:13
    "γιατρός":1426,   # Ιατρική Βεβαίωση {Αρθ. 26 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024)} debugger eval code:6:13
    "κηδεμόνας":1475, # Υπεύθυνη Δήλωση Κηδεμόνα {(Αρθ. 26 της 102791/ΓΔ4 (ΦΕΚ 5130/τ.Β΄/10-09-2024} debugger eval code:6:13
    "γρίπη":1531,     # Λόγω εποχικών ιογενών αναπνευστικών λοιμώξεων
}

absenseTypeId = {
     "0":0,         #  <empty string>
     "Ωριαία":630,  # Προφορική παρατήρηση με ωριαία απομάκρυνση
     "1 μέρα":698,  # Αποβολή από τα μαθήματα μιας (01) ημέρας 
     "2 μέρες":747, # Αποβολή από τα μαθήματα δυο (02) ημερών
     "3 μέρες":796, # Αποβολή από τα μαθήματα τριών (03) ημερών
     "4 μέρες":845, # Αποβολή από τα μαθήματα τεσσάρων (04) ημερών
     "5 μέρες":894, # Αποβολή από τα μαθήματα πέντε (05) ημερών
}

# script για γρήγορη καταχώρηση απουσιών
class js_scripts:
    apousies_prep_script = re.sub(r"\s+"," ","""
 window.FIELDS = {
 DateFrom: "DateFrom",
 Qty:                               "Qty",
 JustifiedQty:                      "JustifiedQty",
 StudentAbsenceJustificationTypeId: "StudentAbsenceJustificationTypeId",
 StudentAbsenceTypeId:              "StudentAbsenceTypeId",
};

window.autoEditQueue = {
    _queue: [],
    _busy: false,

    enqueue(rowIndex, values) {
        this._queue.push({ rowIndex, values });
        this._process();
    },

    _process() {
        if (this._busy || this._queue.length === 0) return;
        this._busy = true;
        const { rowIndex, values } = this._queue.shift();
        this._run(rowIndex, values);
    },

    _run(rowIndex, values) {
        function onEditFormReady(s, e) {
            s.EndCallback.RemoveHandler(onEditFormReady);

            Object.values(FIELDS).forEach(field => {
                if (values[field] != null) s.SetEditValue(field, values[field]);
            });

            s.EndCallback.AddHandler(onUpdateFinished);
            s.UpdateEdit();
        }

        function onUpdateFinished(s, e) {
            s.EndCallback.RemoveHandler(onUpdateFinished);
            autoEditQueue._busy = false;
            autoEditQueue._process();
        }

        gridStudentAbsenceSummary.EndCallback.AddHandler(onEditFormReady);
        if (rowIndex === -1) {
            gridStudentAbsenceSummary.AddNewRow();
        }
        else {
            gridStudentAbsenceSummary.StartEditRow(rowIndex); 
        }
    }
};

window.startAutoEdit = function(rowIndex, values) {
    autoEditQueue.enqueue(rowIndex, values);
};
""")

class MySchool:
    def __init__(self, dl_path, headless):
        self.dl_path = dl_path
        self.st_ids = {}
        js = js_scripts()
        username = get_config("Myschool","Username")
        password = get_config("Myschool","Password")
        firefox_options = Options()
        firefox_options.add_argument("--window-size=1440,900")
        firefox_options.add_argument("--disable-gpu")
        firefox_options.add_argument("--private")
        firefox_options.add_argument("-no-remote") 
        firefox_options.add_argument("--disable-extensions")
        firefox_options.set_preference("browser.download.folderList", 2)
        firefox_options.set_preference("browser.download.dir", str(dl_path))
        if headless: firefox_options.add_argument("--headless")

        self.driver = webdriver.Firefox(
            options=firefox_options,
        )
        self.wait =  WebDriverWait(self.driver, timeout=5, poll_frequency=.2)
        self.driver.delete_all_cookies()
        self.driver.get("https://myschool.sch.gr/")
        self.driver.find_element(By.LINK_TEXT, "Σύνδεση").click()
        WebDriverWait(self.driver, 30).until(lambda d: bool(d.find_element(By.ID, "username")))
        self.driver.find_element(By.ID, "username").send_keys(username)
        WebDriverWait(self.driver, 30).until(lambda d: bool(d.find_element(By.ID, "password")))
        self.driver.find_element(By.ID, "password").send_keys(password)
        WebDriverWait(self.driver, 30).until(lambda d: bool(d.find_element(By.ID, "loginButton")))
        self.driver.find_element(By.ID, "loginButton").click()
        self.report_name = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc, tb):
        self.driver.quit()

    def set_st_ids(self, st_ids):
        self.st_ids = st_ids
        
    def print_cookies(self):
        print(self.driver.get_cookies())
        
    def wait_get_one(self, by_type, by_string):
        try:
            WebDriverWait(self.driver, 60).until(EC.visibility_of_element_located((by_type, by_string)))
            return self.driver.find_element(by_type, by_string)
        except (TimeoutException, NoSuchElementException, StaleElementReferenceException):
            return False

    def wait_get_many(self, by_type, by_string):
        self.wait.until(lambda d: bool(d.find_elements(by_type, by_string)))
        return self.driver.find_elements(by_type, by_string)
    
    def goto(self,report_name):
        self.report_name = report_name
        self.driver.get(report_shortcodes[report_name].url)

    def get_report(self,report_name):
        self.report_name = report_name
        self.driver.get(report_shortcodes[report_name].url)

    def get_page(self,page_name):
        self.driver.get(page_urls[page_name])

    def get_student_page(self, page_name, am):
        self.driver.get(f"{page_urls[page_name]}?id={self.st_ids[am]}")
        
    def provoli(self,provoli):
        self.driver.find_element(By.ID,"ctl00_ContentData_cmbViews_I").click()
        elements = self.driver.find_elements(By.CLASS_NAME, "dxeListBoxItem")
        target = next((el for el in elements if el.text.strip() == provoli), None)
        target.click()

    def select_grades(self,grades):
        for grade in grades:
            self.driver.find_element(By.ID,grade_links[grade]).click()
            WebDriverWait(self.driver, 60).until(EC.element_to_be_clickable((By.ID, "ctl00_ContentData_cbpStudents_lstStudents_LBI1C")))

    def open_extra_criteria(self):
        self.driver.find_element(By.ID,"ctl00_ContentData_btnCriteriaPlus").click()
        iframe = WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.TAG_NAME, "iframe")))
        self.driver.switch_to.frame(iframe)
        WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.XPATH,"//a[normalize-space()='Αποδοχή']")))

    def close_extra_criteria(self):
        self.driver.find_element(By.XPATH,"//a[normalize-space()='Αποδοχή']").click()
        self.driver.switch_to.default_content()

    def set_dates(self,date_from,date_to):
        self.open_extra_criteria()
        from_input = self.wait_get_one(By.ID,"cbpCriteria_dtDateFrom_I")
        from_input.send_keys(date_from)
        to_input = self.wait_get_one(By.ID,"cbpCriteria_dtDateTo_I")
        to_input.send_keys(date_to)
        self.close_extra_criteria()
        
    def click_until(self, elem_id, target_class, max_tries):
        wait = WebDriverWait(self.driver, 30)
        tries = 0
        while tries < max_tries:
            elem = wait.until(EC.element_to_be_clickable((By.ID, elem_id)))
            classes = elem.get_attribute("class").split()
            if target_class in classes:
                break
            elem.click()
            tries += 1
            time.sleep(0.2)
            
    def get_type(self,report_type):
        WebDriverWait(self.driver, 20).until(EC.element_to_be_clickable((By.ID, "ctl00_ContentData_navig_preview")))
        self.driver.find_element(By.ID, "ctl00_ContentData_navig_preview").click()
        filename = report_shortcodes[self.report_name].basename + "." + report_type
        downloadedFile = pathlib.Path(self.dl_path / filename)
        WebDriverWait(self.driver, 60).until(
            EC.visibility_of_element_located((By.ID, "ReportViewer1_ContentFrame"))
        )
        self.driver.execute_script(f"""
var cb = aspxGetControlCollection().Get('ReportToolbar1_Menu_ITCNT13_SaveFormat');
cb.SetValue('{report_type}');
        """)
        self.driver.find_element(By.ID, "ReportToolbar1_Menu_DXI11_").click()
        wait_download_complete(downloadedFile)
        return downloadedFile

    def get_plain(self):
        filename = report_shortcodes[self.report_name].basename + ".xls"
        downloadedFile = pathlib.Path(self.dl_path / filename)
        # if file exists mv to filename.back.n
        self.driver.find_element(By.ID, "ctl00_ContentData_navig_preview").click()
        WebDriverWait(self.driver, 60).until(lambda d: os.path.exists(downloadedFile))
        return downloadedFile

    def get_id_from_am(self,am):
        result = 0
        # wait for it
        inp = self.driver.find_element(By.ID,"ctl00_ContentData_txtRegistrationNo_I")
        inp.send_keys(str(am))
        WebDriverWait(self.driver, 60).until(EC.element_to_be_clickable((By.XPATH,
                                                                         "//a[normalize-space(text())='Αναζήτηση']"
                                        )))
        element = self.driver.find_element(By.XPATH,
                                         "//a[normalize-space(text())='Αναζήτηση']"
                                         )
        element.click()
        wait = WebDriverWait(self.driver, 10)
        try:
            wait.until(
                lambda d: len(self.driver.find_elements(
                    By.XPATH,
                    f"//tr[@id='ctl00_ContentData_gridResults_DXDataRow0']"
                    f"/td[contains(normalize-space(.), '{am}')]"
                )) > 0
            )
            st_row = self.driver.find_element(By.ID,'ctl00_ContentData_gridResults_cell0_0_edit')
            res = st_row.get_property("href").split("=")[-1]
        except TimeoutException:
            res = 0
        element = self.driver.find_element(By.XPATH,
                                         "//a[normalize-space(text())='Καθαρισμός κριτηρίων']")
        element.click()
        return res

    def apousies_prep(self):
        self.driver.execute_script(constants.js_scripts.apousies_prep_script)

    def apousies_get_date(self, dateof):
        xpath = (
            "//tr[contains(@id,'gridStudentAbsenceSummary_DXDataRow')]"
            f"[.//td[contains(normalize-space(.), \"{dateof}\")]]"
        )
        row = WebDriverWait(self.driver, 10).until(
            EC.presence_of_element_located((By.XPATH, xpath))
        )

        row_id = row.get_attribute("id")
        return row_id.split("DXDataRow")[-1]

    def apousies_set(self, rowIndex, params):
        apousies_set_script = f'window.autoEditDone = false; (async () => {{await startAutoEdit({rowIndex}, {{ {params} }}); window.autoEditDone = true;}})();'
        print(apousies_set_script)
        self.driver.execute_script(apousies_set_script)
        WebDriverWait(self.driver, 30).until(
            lambda d: d.execute_script("return window.autoEditDone === true")
        )
        
def get_webdriver(dl_path, headless):
    return MySchool(dl_path, headless)

def get_test_webdriver(dl_path):
    return MySchool(dl_path, headless=False)

def wait_download_complete(path, timeout=120):
    start = time.time()
    last_size = -1
    
    while time.time() - start < timeout:
        if path.exists():
            size = path.stat().st_size
            if size > 0 and size == last_size:
                return
            last_size = size
        time.sleep(2)
    raise TimeoutError(f"Download not completed: {path}")



# βιβλιοθήκη για αποστολή email μέσω σχολικού δικτύου
import smtplib, ssl, time
from gr_sch_helper import sch 
from email.message import EmailMessage
from email.utils import formatdate


port = lambda: sch.get_config('Mail', 'Port')
smtp_server = lambda: sch.get_config('Mail', 'Url')
username = lambda: sch.get_config('Mail', 'Username')                
password = lambda: sch.get_config('Mail', 'Password')
from_mail = lambda: sch.get_config('Mail', 'From')

context = ssl.create_default_context()

# αποστολή απλών μυνημάτων - χωρίς συνημμένα
# το EpalAuto = True αντιστοιχεί σε φίλτρο που
# μεταφέρει τα μυνήματα εκτός inbox
def sendMail(messages):
    #if port = 465
    with smtplib.SMTP(smtp_server(), port(), timeout=30) as server:
        server.ehlo()
        server.starttls(context=context)
        server.ehlo()       
        server.login(username(), password())
        for message in messages:
            msg = EmailMessage()
            msg.set_content(message['body'])
            msg["Date"] = formatdate(localtime=True)
            msg['Subject'] = message['subject']
            msg['From'] = from_mail()
            msg['To'] = message['to']
            msg['Bcc'] = message['bcc']
            msg['EpalAuto']="True"
            # attachment (only pdf / xlsx)
            if message.get("attach"):
                for filename, filetype in message["attach"]:
                    if filetype == "pdf":
                        maintype, subtype = "application", "pdf"
                    elif filetype == "xlsx":
                        maintype, subtype = (
                            "application",
                            "vnd.openxmlformats-officedocument.spreadsheetml.sheet",
                        )
                    else:
                        raise ValueError("Unsupported attachment type")

                    with open(filename, "rb") as f:
                        msg.add_attachment(
                            f.read(),
                            maintype=maintype,
                            subtype=subtype,
                            filename=filename,
                        )
            print("sending to " + message['to'])
            server.send_message(msg)
            print("sent")
            time.sleep(2)

        
# απλή κλάση για να γίνεται αυτόματα το login/logout στον smtp server
# χωρίς να ασχολούμαστε κάθε φορά με τις ρυθμίσεις 
class MyMailer:
    def __enter__(self):
        self.smtp = smtplib.SMTP_SSL(smtp_server(), port(), context=context)
        self.smtp.login(username(),password())
        return self.smtp

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.smtp.quit()
        return False




from gr_sch_helper.sch import get_config

class MyTypst:
    def __init__(self):
        self.SCH_NAME = get_config("Στοιχεία", "όνομα_σχολείου")
        self.SCH_PHONE = get_config("Στοιχεία", "τηλέφωνο")
        self.SCH_ADDR = get_config("Στοιχεία", "διεύθυνση")
        self.SCH_MAIL = get_config("Στοιχεία", "email")

    def before(self, sch_year, date, title):
            return f'''
            #set document(title: [{title}])
            
            #set page(
            margin: (x: 2cm, y: 1cm),
            header: context [
            #set text(size: 8pt)
            #set align(right)
            #counter(page).display("1/1",both: true)
            ],
            )
            
            #set text(lang: "el")
            
            #align(center)[
            #set par(spacing: 0.6em)
            #text(size: 14pt, weight: "bold")[
            {self.SCH_NAME} \\
            ]
            #set par(leading: 0.5em)
            #text(size: 10pt)[
            {self.SCH_ADDR} \\
            Τηλέφωνο: {self.SCH_PHONE} \\
            Email: {self.SCH_MAIL.split('@')[0]}\\@{self.SCH_MAIL.split('@')[1]}
            ]
        ]

        #align(right)[
            #grid(
            columns: (auto, auto),
            gutter:8pt,
            align(right)[#text(size: 8pt)[
            Σχολικό έτος: \\
            Ημερομηνία:]],
            align(right)[#text(size: 8pt)[
            {sch_year} \\
            {date}]]
            )
        ]
        #let student(body) = {{
            set text(size: 10pt, weight: "bold")
            [#body]
            }}
            
            = {title}
'''
